{-# LANGUAGE MultiParamTypeClasses,
             DefaultSignatures,
             FlexibleContexts,
             FlexibleInstances,
             UndecidableInstances,
             OverlappingInstances
 #-}

module EffectCapabilities where

import GHC.Generics

class Channel c_h where
  checkChannel :: c_h -> ()
 
  default checkChannel :: (Generic c_h, GChannel (Rep c_h))
                          => c_h -> ()
  checkChannel c_h = gCheckChannel (from c_h)

class Capability c_p perm where
  checkCapability :: c_p perm -> perm
  getTag          :: c_p perm -> c_p ()
  
  default checkCapability ::  (Generic (c_p perm), GCapability (Rep (c_p perm)) perm) 
                            => c_p perm -> perm
  checkCapability c_p = gCheckCapability (from c_p)

  default getTag :: (Generic (c_p perm), Generic (c_p ()), GGetTag (Rep (c_p perm)) (Rep (c_p ()))) 
                    => c_p perm -> c_p ()
  getTag c_p = to $ gGetTag (from c_p)

class (Channel c_h) => Send c_h c a where 
   receive  :: c_h -> a -> c a

   default receive :: (Generic (c a), GSend c_h a (Rep (c a))) => c_h -> a -> c a
   receive c_h a = to $ gReceive c_h a

--- Generic programming, to avoid explicit definition of instances

class GChannel rep  where
  gCheckChannel :: rep a' -> ()

instance GChannel (M1 i_1 c_1 (M1 i_2 c_2 U1))
    where gCheckChannel (M1 (M1 U1)) = ()

class GCapability rep perm where
  gCheckCapability :: rep a -> perm

instance GCapability (M1 i_1 c_1 (M1 i_2 c_2 (M1 i_3 c_4 (K1 p a)))) a 
    where gCheckCapability (M1 (M1 (M1 (K1 a)))) = a

class GGetTag rep rep' where
  gGetTag :: rep a' -> rep' a'
 
instance GGetTag (M1 i_1 c_1 (M1 i_2 c_2 (M1 i_3 c_4 (K1 p a))))
                 (M1 i_1 c_1 (M1 i_2 c_2 (M1 i_3 c_4 (K1 p ()))))
         where gGetTag (M1 (M1 (M1 (K1 a)))) = M1 (M1 (M1 (K1 ())))

class Channel c_h => GSend c_h a rep where
  gReceive :: c_h -> a -> rep b

instance Channel c_h => GSend c_h a (M1 i_1 c_1 (M1 i_2 c_2 (M1 i_3 c_4 (K1 p a))))
    where gReceive c_h a = checkChannel c_h `seq` M1 (M1 (M1 (K1 a)))

-- To construct a lattice of permissions

class Implies a b where 

instance Implies a a 
instance (Implies a b, Implies b c) => Implies a c


