{-# LANGUAGE TypeOperators,
             FlexibleContexts,
             ScopedTypeVariables,
             MultiParamTypeClasses,
             DeriveGeneric,
             FlexibleInstances
  #-}

module Queue (
       enqueue,
       dequeue,
       dequeueEx,
       dequeueErr,
       QState (),
       QError (),
) where

import Control.Monad.Error
import MonadStatePV
import MonadErrorPV as EPV
import Control.Monad.Error as E
import EffectCapabilities
import GHC.Generics
import {-# SOURCE #-} PriorityQueue
import {-# SOURCE #-} Example

instance Send ExampleChan QError CatchPerm

data QState a = QState a deriving Generic
instance Capability QState a

instance Send PQueueChan QState ReadPerm

data QError a = QError a deriving Generic
instance Capability QError a

enqueue :: MonadStatePV QState n m [s] => s -> m ()
enqueue x = do queue  <- get (QState RWPerm)
               put (QState RWPerm) (queue ++ [x])

dequeue :: MonadStatePV QState n m [s] => m s
dequeue = do queue <- get (QState RWPerm)
             put (QState RWPerm) (tail queue)
             return $ head queue

dequeueErr :: (MonadStatePV QState n m [s], MonadErrorPV QError n' m String) => m s
dequeueErr = EPV.catchError (QError CatchPerm) dequeueEx (\e -> error e)

dequeueEx :: (MonadStatePV QState n m [s], MonadErrorPV QError n' m String) => m s
dequeueEx = do queue <- get (QState ReadPerm)
               if null queue
                  then EPV.throwError (QError ThrowPerm)
                                    "Queue is empty"
                  else do put (QState WritePerm) (tail queue)
                          return $ head queue